# RobotVoice
_Verified Set of TTS Voices for Common Robots_

## Usage
After the setup, as described below, you can print a list of all available robots:

```python
from robotvoice import available_robots
available_robots()
```

To use a robot, you can use the following example code:

- The easiest way to create an existing robot is by using one of the existing and validated voices
```python
from robotvoice import Robot, Robots
robby = Robot(Robots.Reeti)
```

- You can also create a robot by using a voice configuration:
```python
from robotvoice import Robot
config = {
    "voice1": -4,
    "voice2": -6,
    "voice3": 4,
    "voice4": 10,
    "voice5": 12,
    "speaking_speed": 6,
    "effect_name": "vocoder",
    "effect_strength": 9
}
robby = Robot(config=config)
```
- Or you can create a robot by using a json file:
```python
# 3. You can create a robot by giving a json file that holds the configuration:
from robotvoice import Robot
robby = Robot(json_filename="voices/Reeti.json")
```

Once you have a robot, you can let it speak:
```python
robby.speak("Hi, I am Reeti, a very cute robot. How can I help you?")
```

Or store the audio:
```python
robby.write("Hi, I am Reeti, a very cute robot. How can I help you?", "output.wav")
```
Or just synthesize audio and use it further in your code:
```python
audio, samplerate = robby.synthesize("Hi, I am Reeti, a very cute robot. How can I help you?")
```

## Setup

### Run synthesis server
To run locally, install Docker Desktop and run in the folder `RobotVoice`:
```shell
docker-compose up -d --build
```

You should now be able to test it: http://localhost:5000/

### Run synthesis server on a server
Provision at least a m5.xlarge instance on AWS. Install Docker and Docker Compose. Then run:
```shell
git clone https://github.com/polvanrijn/RobotVoice
cd RobotVoice/robotvoice
docker-compose up -d --build
```

If you don't want to use Docker, you can also install the dependencies manually:
```shell
git clone https://github.com/polvanrijn/RobotVoice
cd RobotVoice/robotvoice
sh setup.sh
# Set the port and number of workers based on the number of CPUs, set max-requests = 1 to avoid memory leak
gunicorn server:app --worker-tmp-dir /dev/shm --workers=17 -b :5001 -t 600 --max-requests 1
```

