import time

import torch

import synthesis
import json
import numpy as np
from dsp import Fx
import sounddevice as sd
import soundfile as sf
from enum import Enum


class Robots(Enum):
    Aibo = "Aibo.json"
    Aibo1999 = "Aibo1999.json"
    AILA = "AILA.json"
    Aisoy = "Aisoy.json"
    AlbertHubo = "AlbertHubo.json"
    Alice = "Alice.json"
    AlphaDog = "AlphaDog.json"
    Anafi = "Anafi.json"
    Andrea = "Andrea.json"
    ANYmal = "ANYmal.json"
    AR600 = "AR600.json"
    ARM = "ARM.json"
    Armar = "Armar.json"
    Asimo = "Asimo.json"
    Atlas = "Atlas.json"
    Atlas2013 = "Atlas2013.json"
    Automatronics = "Automatronics.json"
    baerbot = "baerbot.json"
    Bandit = "Bandit.json"
    Baxter = "Baxter.json"
    Baymax = "Baymax.json"
    BEAR = "BEAR.json"
    BHR5 = "BHR5.json"
    BigDog = "BigDog.json"
    Boss = "Boss.json"
    BotVac = "BotVac.json"
    Bruno = "Bruno.json"
    Buddy = "Buddy.json"
    CareObot4 = "CareObot4.json"
    CB2 = "CB2.json"
    Chaos = "Chaos.json"
    CHARLI = "CHARLI.json"
    Charlie = "Charlie.json"
    Chico = "Chico.json"
    Cody = "Cody.json"
    Cog = "Cog.json"
    COMAN = "COMAN.json"
    cozmo = "cozmo.json"
    Curiosity = "Curiosity.json"
    Daisy = "Daisy.json"
    DarwinOP = "DarwinOP.json"
    dash = "dash.json"
    DiegoSan = "DiegoSan.json"
    Digit = "Digit.json"
    dot = "dot.json"
    DRCHubo = "DRCHubo.json"
    Dreamer = "Dreamer.json"
    eBee = "eBee.json"
    ECCE = "ECCE.json"
    Emiew3 = "Emiew3.json"
    EMYS = "EMYS.json"
    Erica = "Erica.json"
    Fable = "Fable.json"
    Fetch = "Fetch.json"
    FLASH = "FLASH.json"
    Flobi = "Flobi.json"
    FROG = "FROG.json"
    Furby = "Furby.json"
    Furhat = "Furhat.json"
    GeminoidDK = "GeminoidDK.json"
    GeminoidF = "GeminoidF.json"
    GeminoidHI = "GeminoidHI.json"
    Genghis = "Genghis.json"
    GlobalHawk = "GlobalHawk.json"
    GoogleSelfDrivingCar = "GoogleSelfDrivingCar.json"
    Handle = "Handle.json"
    Herb = "Herb.json"
    HEXA = "HEXA.json"
    HRP2 = "HRP2.json"
    HRP4 = "HRP4.json"
    HRP4C = "HRP4C.json"
    HRP5P = "HRP5P.json"
    Hubo2 = "Hubo2.json"
    HumanSupportRobot = "HumanSupportRobot.json"
    Husky = "Husky.json"
    HyQ = "HyQ.json"
    iCub = "iCub.json"
    Jaco = "Jaco.json"
    Jibo = "Jibo.json"
    K5 = "K5.json"
    Kaspar = "Kaspar.json"
    Keepon = "Keepon.json"
    KHR3 = "KHR3.json"
    Kibo = "Kibo.json"
    Kilobot = "Kilobot.json"
    Kismet = "Kismet.json"
    Kiwi = "Kiwi.json"
    Kobian = "Kobian.json"
    Kobra = "Kobra.json"
    Kojiro = "Kojiro.json"
    Kuri = "Kuri.json"
    Laikago = "Laikago.json"
    LAURONV = "LAURONV.json"
    LBRiiwa = "LBRiiwa.json"
    LD = "LD.json"
    LegoMindstormsNXT = "LegoMindstormsNXT.json"
    Leonardo = "Leonardo.json"
    Lola = "Lola.json"
    M1 = "M1.json"
    Mabu = "Mabu.json"
    Mahru = "Mahru.json"
    MamboFPV = "MamboFPV.json"
    Meca500 = "Meca500.json"
    Milo = "Milo.json"
    MiniCheetah = "MiniCheetah.json"
    MistyII = "MistyII.json"
    Nao = "Nao.json"
    NaviShaman = "NaviShaman.json"
    Nextage = "Nextage.json"
    nuTonomy = "nuTonomy.json"
    Octavia = "Octavia.json"
    Olivia = "Olivia.json"
    OTTO = "OTTO.json"
    PackBot = "PackBot.json"
    Paro = "Paro.json"
    Partner = "Partner.json"
    Pepper = "Pepper.json"
    Perseverance = "Perseverance.json"
    Petman = "Petman.json"
    Photon = "Photon.json"
    Pleo = "Pleo.json"
    Pneuborn = "Pneuborn.json"
    PR2 = "PR2.json"
    Qbo = "Qbo.json"
    Qrio = "Qrio.json"
    QTrobot = "QTrobot.json"
    QTrobot2 = "QTrobot2.json"
    Quince = "Quince.json"
    Raven = "Raven.json"
    REEMB = "REEMB.json"
    REEMC = "REEMC.json"
    Reeti = "Reeti.json"
    RHex = "RHex.json"
    RoboBee = "RoboBee.json"
    Robonaut2 = "Robonaut2.json"
    RoboThespian = "RoboThespian.json"
    Roboy = "Roboy.json"
    RollinJustin = "RollinJustin.json"
    Roomba = "Roomba.json"
    Root = "Root.json"
    Rosie = "Rosie.json"
    RVR = "RVR.json"
    SalamandraRoboticaII = "SalamandraRoboticaII.json"
    Sawyer = "Sawyer.json"
    Schlupp = "Schlupp.json"
    Simon = "Simon.json"
    SmartBird = "SmartBird.json"
    Sophia = "Sophia.json"
    Sphero = "Sphero.json"
    SpiritOpportunity = "SpiritOpportunity.json"
    Spot = "Spot.json"
    Starship = "Starship.json"
    Stuntronics = "Stuntronics.json"
    Surena = "Surena.json"
    TALOS = "TALOS.json"
    Telenoid = "Telenoid.json"
    Throwbot = "Throwbot.json"
    TIAGo = "TIAGo.json"
    Titan = "Titan.json"
    TORO = "TORO.json"
    TUlip = "TUlip.json"
    TURTLE = "TURTLE.json"
    TwendyOne = "TwendyOne.json"
    UnicornBot = "UnicornBot.json"
    Valkyrie = "Valkyrie.json"
    Vector = "Vector.json"
    Versatrax = "Versatrax.json"
    VIVA = "VIVA.json"
    Wabot2 = "Wabot2.json"
    Wakamaru = "Wakamaru.json"
    WAM = "WAM.json"
    Waymo = "Waymo.json"
    YuMi = "YuMi.json"
    Zeno = "Zeno.json"
    Zipline = "Zipline.json"


def available_robots():
    list = [e for e in Robots]
    for name in list:
        print(name)
    return list
class Robot:
    def __init__(self, robot=None, json_filename=None, config=None):
        if robot != None:
            self.config = json_to_config("voices/" + robot.value)
        elif json_filename != None:
            self.config = json_to_config(json_filename)
        elif config != None:
            self.config = config
        else:
            print("You must specify either a robot name, a json file, or a config dictionary! To list all available robots, use robotvoice.available_robots()")
        self.speaker_embeddings = synthesis.convert_weights_to_speaker_embedding(np.array([self.config["voice1"],self.config["voice2"],self.config["voice3"],self.config["voice4"],self.config["voice5"]],dtype=np.float32).reshape(1, -1), 5, "PCA")[0]
        self.net_g, self.hps = synthesis.load_model()

        data = {'speed': int(self.config["speaking_speed"])}
        self.voice_effects_dict_list =  synthesis.get_effects_dict(data, synthesis.SUPPORTED_VOICE_EFFECTS, 12, zero_idx=7)
        #


        self.initialize_effects()

    def initialize_effects(self):
        self.voice_effects = synthesis.get_current_effect_values(self.voice_effects_dict_list, 0)

        data = {}
        data["flanger"] = 0
        data['flanger_type'] = 1
        data['vocoder_type'] = 1
        data['griffin'] = 0
        data['pitch'] = 0
        data['tremolo'] = 0
        data['timeshift'] = 0
        data['vocoder'] = 0

        key = str(self.config['effect_name'])
        if key.startswith('flanger'):
            data['flanger_type'] = key[-1]
            data['flanger'] = int(self.config['effect_strength'])
        elif key.startswith('vocoder'):
            data['vocoder_type'] = 2
            data['vocoder'] = int(self.config['effect_strength'])
        else:
            data[key] = int(self.config['effect_strength'])


        self.robot_effects_dict_list = synthesis.get_effects_dict(data, synthesis.SUPPORTED_ROBOT_EFFECTS, 12)
        self.robot_effects = synthesis.get_current_effect_values(self.robot_effects_dict_list,0)
        self.robot_effects, self.additional_parameters = synthesis.update_parameters(self.robot_effects)

        pass

    def synthesize(self, text):
        text = text
        stn_tst = synthesis.get_text(text, self.hps)
        speaker_embedding = self.speaker_embeddings
        spk_emb = torch.from_numpy(speaker_embedding.astype(np.float32))[None]
        audio, sr = synthesis.tts(self.net_g, self.hps, stn_tst, spk_emb)
        audio, sr = synthesis.apply_voice_effects(audio, sr, self.voice_effects)
        audio = Fx(sr).process_audio(audio, self.robot_effects, self.additional_parameters)
        audio = synthesis.normalize(audio)
        return audio, sr

    def speak(self, text):
        audio, sr = self.synthesize(text)

        #Pad the signal - otherwise, as the sounddevice playback functionality often cuts the signal at the beginning and end
        audio = np.pad(audio,(200,5000), 'constant')
        sd.play(audio, sr)
        sd.wait()


    def write(self, text, filename):
        audio, sr = self.synthesize(text)
        sf.write(filename, audio, sr)

def json_to_config(json_filename):
    # Opening JSON file
    f = open(json_filename)
    c = json.load(f)
    return c


